//---------------------------------------------------------------------------
// Copyright (C) 2000 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
//  initcopr.c - This utility uses TMEX to initialize the SHA iButton 
//              (DS1963S), coprocessor.
//
//  Compiler:  Microsoft Visual C 
//  Version:   1.02
//  History:   
//

#define TMEXUTIL

#include "ibtmexcw.h"   
#include "ibsha.h"

// external function prototypes 
extern int RunScriptSHA(Script script[], void *, int);
extern int PrintHeader(char *);
extern void output_status(int, char *);
extern int EnterString(char *, char *, int, int);
extern int ToHex(char ch);
extern int GetSeviceProviderSettings(int, char *, uchar *, int, uchar *);
extern int WriteFileSHA(uchar *, uchar *, int, uchar *);
extern int bitacc(int, int, int, uchar *);
extern void ExitProg(char *, int);

// defines
#define EXIT_SUCCESS             0
#define EXIT_ERROR               1
#define EXIT_ERROR_WRITE         2
#define EXIT_ERROR_MONEY_SCRT    3
#define EXIT_ERROR_AUTH_SCRT     4
#define EXIT_ERROR_NO_SHA        5


//----------------------------------------------------------------------
//  This is the Main routine for initcopr 
//
void main(short argc, char **argv)
{
   int ShowStatus=FALSE,filenum;
   char msg[200];
   uchar auth_secret[65],money_secret[65];
   int cnt,exit_code,i;
   uchar buf[6],flname[6],page_map[1];

   // check arguments to see if request instruction with '?' or too many
   if ((argc > 3) || ((argc > 1) && (argv[1][0] == '?' || argv[1][1] == '?')))
       ExitProg("\nusage: initcopr <output_filename> </Verbose>\n"
              "  - initialize a coprocessor SHA iButton on default\n"
              "    1-Wire Net port (use SetPrt32.exe to set)\n"
              "  - <output_filename> optional output filename\n"
              "  - </V> optional show verbose output (default not shown)\n"
              "  - version 1.02\n",EXIT_ERROR);

   // read the default PortNum and PortType 
   if (!TMReadDefaultPort(&PortNum[IB_ROVING],&PortType[IB_ROVING]))
      ExitProg("ERROR, Could not read the default PortNum and PortType!\n",EXIT_ERROR);

   // hard code CO_PROCESSOR port to be the same
   PortNum[IB_COPR] = PortNum[IB_ROVING];
   PortType[IB_COPR] = PortType[IB_ROVING];

   // print the header info 
   if (!PrintHeader("InitCoPr Version 1.02"))
      ExitProg("No driver found\n",EXIT_ERROR);

   // check arguments 
   VERBOSE = FALSE;
   filenum = 0;
   if (argc >= 2)
   {
      if (argv[1][0] != '/')
         filenum = 1;
      else if ((argv[1][1] == 'V') || (argv[1][1] == 'v'))
         VERBOSE = TRUE;

      if (argc == 3)
      {    
         if (argv[2][0] != '/')
            filenum = 2;
         else if ((argv[2][1] == 'V') || (argv[2][1] == 'v'))
            VERBOSE = TRUE;
      }   
   }

   // open the output file  
   fp = NULL;
   if (filenum > 0)
   {
      fp = fopen(argv[filenum],"wb");
      if(fp == NULL)
      {    
         printf("ERROR, Could not open output file!\n");
         exit(EXIT_ERROR);
      }
      else
         printf("File '%s' opened to write sensor data.\n\n",
                 argv[filenum]);
   }

   // set the default pages used for secrets (will be written to copr)
   c_mmaster_scrt = 0;  // co-processor money master secret
   c_amaster_scrt = 1;  // co-processor auth master secret
   c_udevice_scrt = 2;  // co-processor unique device secret(calculated)
   r_udevice_scrt = 5;  // roving unique device secret
   t_udevice_scrt = 4;  // till unique device secret

   // run the script to setup the 1-Wire
   if (RunScriptSHA(Verify1Wire, NULL, 0))
      output_status(LV_ALWAYS,"1-Wire port verified\n\n");
   else
      ExitProg("1-Wire Net not successfuly setup, end program\n",EXIT_ERROR);

   // prompt for service provider settings
   output_status(LV_ALWAYS,"Enter information to create the co-processor SHA"
                " iButton for a service\n");
   if (!GetSeviceProviderSettings(TRUE, provider,auth_secret,TRUE, money_secret))
      ExitProg("Aborted entering the provider settings, end program\n",EXIT_ERROR);

   // create the data for the TMEX file
   sprintf(flname,"%s",copr_file);   // first 4 letters, service name
   sprintf(buf,"%s",provider);
   buf[4] = 102;                 // extension of SHA money file
   buf[5] = c_mmaster_scrt;      // money master secret number
   buf[6] = c_amaster_scrt;      // auth master secret number
   buf[7] = c_udevice_scrt;      // unique device secret(calculated)
   page_map[0] = 1;              // page map for only 1 file page
  
   // get warning
   output_status(LV_ALWAYS,"\n** Warning SHA iButton on 1-Wire will be overwritten!!\n  (Press any key)\n");
   MessageBeep(MB_OK);
   getch();
   
   // attempt to open session on 1-Wire net
   if (RunScriptSHA(OpenSession, NULL, 0))
   {
      output_status(LV_ALWAYS,"\n\n** Session on 1-Wire opened\n");

      // select co-processor port
      if (RunScriptSHA(SelectCoProcessor, NULL, 0))
      {
         output_status(LV_ALWAYS,"** 1-Wire port for co-processor selected\n");

         // find the first available SHA iButton
         if (RunScriptSHA(FindSHA, &copr_rom[0], sizeof(copr_rom)))
         {
            // loop to print ROM of device found
            cnt = sprintf(msg,"** SHA iButton found: ");
            for (i = 7; i >= 0; i--)
               cnt += sprintf(msg + cnt,"%02X",copr_rom[i]);
            sprintf(msg + cnt,"\n");
            output_status(LV_ALWAYS,msg);

            // create the authentication secret
            if (RunScriptSHA(CreateAuthSecret, &auth_secret[0], 32))
            {
               output_status(LV_ALWAYS,"** Authenticate secret created\n");

               // create the money secret
               if (RunScriptSHA(CreateMoneySecret, &money_secret[0], 32))
               {
                  output_status(LV_ALWAYS,"** Money secret created\n");
                  // coprocessor is already current device
                  if (WriteFileSHA(flname, buf, 8, page_map))
                  {
                     output_status(LV_ALWAYS,"** Co-processor info file created\n");
                     exit_code = EXIT_SUCCESS;
                  }
                  else
                  {
                     output_status(LV_ALWAYS,"ERROR, Could not write Co-processor info file\n");
                     exit_code = EXIT_ERROR_WRITE;
                  }
               }
               else
               {
                  output_status(LV_ALWAYS,"ERROR, Could not create money secret\n");
                  exit_code = EXIT_ERROR_MONEY_SCRT;
               }
            }
            else
            {
               output_status(LV_ALWAYS,"ERROR, Could not create authorization secret\n");
               exit_code = EXIT_ERROR_AUTH_SCRT;
            }
         }
         else
         {
            output_status(LV_ALWAYS,"ERROR, Could not find a SHA iButton\n");
            exit_code = EXIT_ERROR_NO_SHA;
         }
      }
      else
      {
         output_status(LV_ALWAYS,"ERROR, Could not select co-processor 1-Wire\n");
         exit_code = EXIT_ERROR;
      }
   }
   else
   {
      output_status(LV_ALWAYS,"ERROR, Could not open co-processor 1-Wire port\n");
      exit_code = EXIT_ERROR;
   }

   // close session
   if (RunScriptSHA(CloseSession, NULL, 0))
      output_status(LV_ALWAYS,"** Session on 1-Wire closed\n");

   // close opened file
   if ((fp != NULL) && (fp != stdout))
   {
      printf("File '%s' closed.\n",
              argv[filenum]);
      fclose(fp);
   }

   // check on result of operation
   if (exit_code)
      printf("ERROR reported (%d), end program\n",exit_code);
   else
      printf("End program normally\n");

   // end the program
   exit(exit_code);
}


